﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.Win32;

namespace Hangar
{
    public partial class MainForm : Form
    {
        private bool unsaved;
        private string openFile;

        public MainForm()
        {
            InitializeComponent();

            unsaved = false;
            toolStripStatusLabel1.Text = "";

            using (RegistryKey myKey = Registry.CurrentUser
                    .CreateSubKey(@"Software\Hangar",
                        RegistryKeyPermissionCheck.ReadSubTree))
            {
                openFile = myKey.GetValue("lastOpen") as String;
            }

            if (openFile != null)
            {
                Load(openFile);
            }

            Paint += (sender, e) =>
                {
                    Aircraft selectedAircraft =
                        aircraftBox.SelectedItem as Aircraft;

                    if (selectedAircraft != null)
                    {
                        e.Graphics.FillRectangle(
                            new SolidBrush(selectedAircraft.Colour),
                            new Rectangle(410, 222, 50, 20));
                    }
                };
        }

        private void UpdateStatus()
        {
            toolStripStatusLabel2.Text = String.Format
                (
                    "Number of Aircrafts: {0}",
                    aircraftBox.Items.Count
                );
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AircraftDialog diag = new AircraftDialog();

            if (diag.ShowDialog() == DialogResult.OK)
            {
                aircraftBox.Items.Add(diag.Aircraft);
                unsaved = true;
                toolStripStatusLabel1.Text = "Unsaved Changes";
                UpdateStatus();
            }
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFile != null)
            {
                Save(openFile);
            }
            else
            {
                SaveAs();
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveAs();
        }

        private void Save(string fileName)
        {
            try
            {
                using (StreamWriter output =
                    new StreamWriter(fileName))
                {
                    foreach (Aircraft aircraft in
                        aircraftBox.Items.Cast<Aircraft>())
                    {
                        output.WriteLine(String.Format(
                            "{0};{1};{2};{3};{4};{5};{6}",
                            aircraft.Manufacturer,
                            aircraft.TailNumber,
                            aircraft.Range,
                            aircraft.Colour.A,
                            aircraft.Colour.R,
                            aircraft.Colour.G,
                            aircraft.Colour.B));
                    }
                }

                unsaved = false;
                toolStripStatusLabel1.Text = "";
                //output.Close();
            }
            catch (IOException ex)
            {
                MessageBox.Show(String.Format(
                    "Failed to save database due to {0}",
                    ex.Message), "Error");
            }
        }

        private void SaveAs()
        {
            SaveFileDialog diag = new SaveFileDialog();

            diag.DefaultExt = "hdb";
            diag.Filter = "Hangar Database|*.hdb";
            diag.InitialDirectory = Environment.CurrentDirectory;
            diag.Title = "Select database file";

            if (diag.ShowDialog() == DialogResult.OK)
            {
                Save(diag.FileName);
                openFile = diag.FileName;
            }
        }

        private void Load(string fileName)
        {
            aircraftBox.Items.Clear();

            try
            {
                using (StreamReader input =
                    new StreamReader(fileName))
                {
                    string line;
                    while ((line = input.ReadLine()) != null)
                    {
                        string[] atributes = line.Split(';');

                        Aircraft aircraft = new Aircraft();

                        aircraft.Manufacturer = atributes[0];
                        aircraft.TailNumber = atributes[1];

                        int range;
                        if (Int32.TryParse(atributes[2], out range))
                        {
                            aircraft.Range = range;
                            aircraft.Colour = Color.FromArgb(
                                Int32.Parse(atributes[3]),
                                Int32.Parse(atributes[4]),
                                Int32.Parse(atributes[5]),
                                Int32.Parse(atributes[6]));

                            aircraftBox.Items.Add(aircraft);
                        }
                    }
                }

                unsaved = false;
                toolStripStatusLabel1.Text = "";
                UpdateStatus();
            }
            catch (IOException ex)
            {
                MessageBox.Show(String.Format(
                    "Failed to load database due to {0}",
                    ex.Message), "Error");
            }
        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (CheckUnsaved())
            {
                OpenFileDialog diag = new OpenFileDialog();

                diag.Filter = "Hangar Database|*.hdb";//|All Files|*.*";
                diag.InitialDirectory = Environment.CurrentDirectory;

                if (diag.ShowDialog() == DialogResult.OK)
                {
                    Load(diag.FileName);
                    openFile = diag.FileName;
                }
            }
        }

        private void aircraftBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            Aircraft selectedAircraft =
                aircraftBox.SelectedItem as Aircraft;

            if (selectedAircraft == null)
            {
                manufacturerLabel.Text = "Manufacturer:";
                tailNumberLabel.Text = "Tail Number:";
                rangeLabel.Text = "Range: km";

                removeButton.Visible = false;
            }
            else
            {
                manufacturerLabel.Text = 
                    String.Format("Manufacturer: {0}",
                    selectedAircraft.Manufacturer);

                tailNumberLabel.Text =
                    String.Format("Tail Number: {0}",
                    selectedAircraft.TailNumber);

                rangeLabel.Text = String.Format(
                    "Range: {0} km",
                    selectedAircraft.Range);

                removeButton.Visible = true;
            }

            Invalidate();
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            aircraftBox.Items.Remove(aircraftBox.SelectedItem);

            unsaved = true;
            toolStripStatusLabel1.Text = "Unsaved Changes";
            UpdateStatus();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (openFile != null)
            {
                using (RegistryKey myKey = Registry.CurrentUser
                    .CreateSubKey(@"Software\Hangar",
                        RegistryKeyPermissionCheck.ReadWriteSubTree))
                {
                    myKey.SetValue("lastOpen", openFile);
                }
            }

            if (!CheckUnsaved())
            {
                e.Cancel = true;
            }
        }

        private bool CheckUnsaved()
        {
            if (unsaved)
            {
                DialogResult userChoice = MessageBox.Show(
                    "Save changes before quitting?",
                    "unsaved changes",
                    MessageBoxButtons.YesNoCancel,
                    MessageBoxIcon.Question);

                switch (userChoice)
                {
                    case DialogResult.Yes:
                        {
                            //saveToolStripMenuItem.PerformClick();
                            //saveToolStripMenuItem_Click(this, new EventArgs());
                            SaveAs();
                            break;
                        }
                    case DialogResult.No:
                        {
                            break;
                        }
                    default:
                        {
                            return false;
                        }
                }
            }

            return true;
        }
    }
}
